SystemOrganization addCategory: #'Seaside-Scaffolding'!
SystemOrganization addCategory: #'Seaside-Scaffolding-Model'!
SystemOrganization addCategory: #'Seaside-Scaffolding-Test'!

TestCase subclass: #SGTestCase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Scaffolding-Test'!

!SGTestCase methodsFor: 'private' stamp: 'obi 6/13/2010 19:15'!
classNamed: aName
	^SGSystem classNamed: aName! !

!SGTestCase methodsFor: 'private' stamp: 'obi 11/07/2008 06:49'!
createTestClass
	| aClass |
	aClass := SGModel subclass: #SGTestA instVarNames: #(name description counter).
	aClass descriptionAt: 3 put: (aClass integerDescriptionOn: #counter).
	^aClass! !

!SGTestCase methodsFor: 'private' stamp: 'obi 11/07/2008 06:07'!
executeWithTestClass: aBlock
	[aBlock value: self createTestClass] ensure: [self removeTestClass]! !

!SGTestCase methodsFor: 'private' stamp: 'obi 10/28/2008 13:30'!
removeClass: aClass
	aClass isNil ifFalse: [
		aClass safelyDo: [:i | i discard].
		self assert: (SGSystem removeClassNamed: aClass name)].! !

!SGTestCase methodsFor: 'private' stamp: 'obi 10/28/2008 13:33'!
removeTestClass
	self removeClass: self testClass! !

!SGTestCase methodsFor: 'test-components' stamp: 'obi 10/28/2008 13:34'!
testAsComponent
	| newObject |
	self createTestClass.
	newObject := self testClass new.
	newObject asComponent.
	self removeTestClass! !

!SGTestCase methodsFor: 'private' stamp: 'obi 11/03/2008 13:54'!
testClass
	^self classNamed: #SGTestA! !

!SGTestCase methodsFor: 'test-descriptions' stamp: 'obi 10/28/2008 12:26'!
testDescription
	| aContainer |
	aContainer := SGModel description.
	self assert: (aContainer isKindOf: MAContainer).
	self assert: aContainer children isEmpty! !

SGTestCase subclass: #SGTestInstance
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Scaffolding-Test'!

!SGTestInstance methodsFor: 'testing' stamp: 'obi 11/07/2008 06:52'!
testInstanceAttributeChange
	self executeWithTestClass:
		[:aClass | | newObject |
		newObject := aClass new.
		newObject name: 'Test'.
		newObject description: 'Description'.
		newObject counter: 10.
		self assert: (newObject name = 'Test').
		self assert: (newObject description = 'Description').
		self assert: (newObject counter = 10)]! !

!SGTestInstance methodsFor: 'testing' stamp: 'obi 11/07/2008 06:50'!
testInstanceAttributeInitialize
	self executeWithTestClass:
		[:aClass | | newObject |
		newObject := aClass new.
		self assert: (newObject name isNil).
		self assert: (newObject description isNil).
		self assert: (newObject counter = 0)]! !

!SGTestInstance methodsFor: 'testing' stamp: 'obi 11/07/2008 05:59'!
testInstanceCreating
	| anObject oldSize newSize|
	oldSize := SGModel allInstances size.
	anObject := SGModel new.
	
	self assert: (anObject isKindOf: SGModel).
	self assert: oldSize + 1 == SGModel allInstances size.
	anObject discard.
	self assert: oldSize == SGModel allInstances size.! !

!SGTestInstance methodsFor: 'testing' stamp: 'obi 05/05/2009 13:36'!
testTwowayRelation
	| aUser aUser2 aGroup |
	aUser := SGUser new.
	aUser loginName: 'TestUser_1'.
	aUser2 := SGUser new.
	aUser2 loginName: 'TestUser_2'.
	aGroup := SGGroup new.
	aGroup name: 'TestGroup'.
	aUser groupsAdd: aGroup.
	aGroup usersAdd: aUser2.
	self assert: aGroup users size = 2.
	self assert: aUser groups size = 1.
	self assert: aUser2 groups size = 1.
	aUser discard.
	self assert: aGroup users size = 1.
	aUser2 discard.
	self assert: aGroup users size = 0.
	aGroup discard! !

SGTestCase subclass: #SGTestMigration
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Scaffolding-Test'!

!SGTestMigration methodsFor: 'testing' stamp: 'obi 6/13/2010 20:16'!
testClassCreation
	| testClassName newClass newObject |
	testClassName := #SGTestA.
	newClass := SGModel subclass: testClassName instVarNames: #(name description).
	self assert: newClass notNil.
	newObject := newClass new.
	newObject name: 'Test'; description: 'Description'.
	self assert: newObject name = 'Test'.
	self assert: newClass description notNil.
	self assert: (newClass attributeDescriptions size == 2).
	newClass safelyDo: [:i | i discard].
	self assert: (SGSystem removeClassNamed: testClassName).
	self assert: (SGSystem classNamed: testClassName) isNil.
	self assert: (SGSystem hasInstancesOf: testClassName) not.! !

!SGTestMigration methodsFor: 'testing' stamp: 'obi 6/13/2010 20:18'!
testClassRedefine
	| testClassName newClass newObject newObject2 |
	testClassName := #SGTestA.
	newClass := SGModel subclass: testClassName instVarNames: #(name description).
	newClass attributeContainer.
	newObject := newClass new.
	newClass := SGModel subclass: testClassName instVarNames: #(name description label).
	self assert: (newClass attributeDescriptions size == 3).
	newObject2 := newClass new.
	newObject name: 'Test'; description: 'Description'; label: 'Label'.
	newObject2 name: 'Test2'; description: 'Description2'; label: 'Label2'.
	self assert: newClass meta keys notEmpty.
	self assert: newObject class = newObject2 class.
	self assert: newObject name = 'Test'.
	self assert: newObject2 name = 'Test2'.
	newClass safelyDo: [:i | i discard].
	self assert: (SGSystem removeClassNamed: testClassName).
	self assert: (SGSystem classNamed: testClassName) isNil.
	self assert: (SGSystem hasInstancesOf: testClassName) not.! !

!SGTestMigration methodsFor: 'testing' stamp: 'obi 6/13/2010 20:18'!
testClassRedefine2
	| testClassName newClass newObject newObject2 |
	testClassName := #SGTestA.
	newClass := SGModel subclass: testClassName instVarNames: #(name description label).
	newClass attributeContainer.
	newObject := newClass new.
	newClass := SGModel subclass: testClassName instVarNames: #(name description).
	self assert: (newClass attributeDescriptions size == 2).
	newObject2 := newClass new.
	newObject name: 'Test'; description: 'Description'.
	newObject2 name: 'Test2'; description: 'Description2'.
	self assert: newClass meta keys notEmpty.
	self assert: newObject class = newObject2 class.
	self assert: newObject name = 'Test'.
	self assert: newObject2 name = 'Test2'.
	newClass safelyDo: [:i | i discard].
	self assert: (SGSystem removeClassNamed: testClassName).
	self assert: (SGSystem classNamed: testClassName) isNil.
	self deny: (SGSystem hasInstancesOf: testClassName).! !

!SGTestMigration methodsFor: 'testing' stamp: 'obi 6/13/2010 20:17'!
testSuperclassRedefine
	| testClassName testClassName2 newClass newClass2 newObject newObject2 |
	testClassName := #SGTestA.
	testClassName2 := #SGTestA1.
	newClass := SGModel subclass: testClassName instVarNames: #(name description).
	newObject := newClass new.
	newObject name: 'Test A'.
	newClass2 := newClass subclass: testClassName2 instVarNames: #(test1 test2).
	newObject := newClass2 new.
	newObject name: 'Test A1'.
	newClass := SGModel subclass: testClassName instVarNames: #(name).
	
	"reassign class"
	newClass2 := SGSystem classNamed: testClassName2.
	
	self assert: newClass2 allInstVarNames size == 3.
	newClass safelyDo: [:i | i discard].
	newClass2 safelyDo: [:i | i discard]. 
	self assert: (SGSystem removeClassNamed: testClassName2).
	self assert: (SGSystem removeClassNamed: testClassName).
	self assert: (SGSystem classNamed: testClassName) isNil.
	self assert: (SGSystem classNamed: testClassName2) isNil.
	self assert: (SGSystem hasInstancesOf: testClassName) not.
	self assert: (SGSystem hasInstancesOf: testClassName2) not.! !

MAOneToManyComponent subclass: #SGTwowayRelationComponent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Scaffolding'!

!SGTwowayRelationComponent class methodsFor: 'testing' stamp: 'obi 05/12/2009 10:59'!
isAbstract
	^ false! !

!SGTwowayRelationComponent methodsFor: 'actions' stamp: 'obi 05/16/2009 11:21'!
add
	| newModel result |
	newModel := self selected new.
	result := self root call: newModel open.
	result isNil
		ifFalse: [self add: newModel; refresh]
		ifTrue: [newModel discard]! !

!SGTwowayRelationComponent methodsFor: 'actions' stamp: 'obi 05/16/2009 10:55'!
add: anObject
	self description accessor add: anObject to: self model.
	"self refresh"! !

!SGTwowayRelationComponent methodsFor: 'private' stamp: 'obi 05/12/2009 12:41'!
buildCommands
	commands := MACommandColumn new setReport: self report.
	self description isDefinitive ifFalse: [
		commands
			addCommandOn: self selector: #edit: text: 'edit';
			addCommandOn: self selector: #remove: text: 'remove' ].
	^ commands! !

!SGTwowayRelationComponent methodsFor: 'private' stamp: 'obi 05/12/2009 12:37'!
buildReport
	| newReport |
	newReport := (MAReport rows: self value)
					sortEnabled: self description isOrdered not.
	self reference noneRelationDescriptions do: [:desc | newReport addColumnDescription: desc ].
	^newReport! !

!SGTwowayRelationComponent methodsFor: 'private' stamp: 'obi 05/12/2009 12:49'!
model
	^self memento model! !

!SGTwowayRelationComponent methodsFor: 'actions' stamp: 'obi 05/12/2009 12:49'!
remove: anObject
	self description accessor remove: anObject from: self model.
	self refresh! !

!SGTwowayRelationComponent methodsFor: 'rendering' stamp: 'obi 05/16/2009 11:22'!
renderEditorOn: html
	 self renderSelectListOn: html.
	self renderViewerOn: html.! !

MASelectorAccessor subclass: #SGTwowayRelationAccessor
	instanceVariableNames: 'targetSelector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Scaffolding'!

!SGTwowayRelationAccessor class methodsFor: 'accessing' stamp: 'obi 05/05/2009 13:27'!
relationClass
	^OrderedCollection! !

!SGTwowayRelationAccessor methodsFor: 'model' stamp: 'obi 05/12/2009 13:08'!
add: anObject to: aModel
	| aColl |
	aColl := aModel perform: self readSelector.
	aColl isNil ifTrue: [super write: (aColl := self class relationClass new) to: aModel].
	(aColl includes: anObject)
		ifFalse: [
			aColl add: anObject.
			(anObject perform: targetSelector) add: aModel]
	
	
	! !

!SGTwowayRelationAccessor methodsFor: 'model' stamp: 'obi 05/16/2009 12:24'!
addAll: aCollection to: aModel
	aCollection do: [:each | self add: each to: aModel]
	! !

!SGTwowayRelationAccessor methodsFor: 'model' stamp: 'obi 05/05/2009 13:39'!
remove: anObject from: aModel
	"remove the objects from both collections"
	(aModel perform: self readSelector) remove: anObject ifAbsent: [].
	(anObject perform: targetSelector) remove: aModel ifAbsent: []! !

!SGTwowayRelationAccessor methodsFor: 'model' stamp: 'obi 05/05/2009 13:30'!
removeAllFrom: aModel
	(aModel perform: readSelector) do: [:each | self remove: each from: aModel]! !

!SGTwowayRelationAccessor methodsFor: 'accessing' stamp: 'obi 04/21/2009 13:28'!
targetSelector 
	^targetSelector! !

!SGTwowayRelationAccessor methodsFor: 'accessing' stamp: 'obi 04/21/2009 13:27'!
targetSelector: aSymbol
	targetSelector := aSymbol! !

!SGTwowayRelationAccessor methodsFor: 'model' stamp: 'obi 05/24/2009 10:32'!
write: anObject to: aModel
	| aColl |
	"add anObject to my collection and also add aModel to the other object collection 2"
	anObject ifNil: [^self removeAllFrom: aModel].
	(anObject isKindOf: Collection)
		ifTrue: [super write: anObject to: aModel]
		ifFalse: [self add: anObject to: aModel]! !

WAFileLibrary subclass: #SGLibrary
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Scaffolding'!

!SGLibrary methodsFor: 'other' stamp: 'obi 11/07/2008 13:57'!
adminCss
	^'
	body {margin: 0px;padding: 10px;border: 0px;background: gray;text-align: center;}
	body, div, tr, td, th, form input, form submit {font-family: verdana, Arial, Helvetica, sans-serif;font-size: 11px;}
	table, tr, td, th, form {margin: 0px;padding: 0px;border: 0px;}
	th {text-align: left;}
	a, a:visited {color: #000046;text-decoration: none;}
	.container {margin: auto;width: 980px;background: white ;border: 1px solid #24a;border-top: 5px solid #24a;border-bottom: 5px solid #24a;text-align: left;-moz-border-radius:5px;border-radius:5px}
	.top {width: 880px; height: 50px;background:#c5c4c3;border-bottom: 1px solid black;font-size: 9px;text-align: right;}
	.toolbar {height: 18px;border-bottom: 1px dotted black;border-top: 1px dotted black;background: white;}
	.toolbar a {padding-left: 10px;padding-right: 10px;padding-top: 3px;padding-bottom: 2px;margin-left: -1px;border-left: 1px solid black;border-right: 1px solid black;line-height: 18px;letter-spacing: 1px;text-decoration: none;}
	.toolbar a:hover {color: white;background-color: #000046;}
	.page {padding: 0px;width:100%}
	.header {padding-bottom:4px;padding-left:4px;background:#9cf}
	.page td,.page th {padding-right: 5px;vertical-align: top;}
	.page .navigation {width: 110px;padding: 6px;padding-top: 0px;border-right:1px dotted #ccc}
	.page .navigation .title {margin-top: 4px;font-weight: bold;}
	.page .contents {width: 830px;padding-left:6px;padding-top:4px; height: 600px;text-align: justify;overflow:auto}
	.page .contents a {color:#5588ff}
	.page .contents a:hover {text-decoration:underline}
	.page .contents td {padding:2px 1px}

	h1,h2,h3 {margin-top: 0px}
	h1 {margin-bottom: 2px;font-size: 24px;}
	h2 {margin-bottom: 1px;font-size: 15px;}
	h3 {margin-bottom: 1px;font-size: 13px;}
	.page h1 {margin-bottom: 3px;font-size: 15px;}
	.page h2 {margin-bottom: 2px;font-size: 12px;}
	.page h3 {margin-top: 4px;margin-bottom: 2px;font-size: 12px;}
	.tree ul {margin-left:6px;padding-left:8px}
	hr {border: none;border-bottom: 1px dotted black;}
	.bottom {padding-left: 5px;padding-right: 5px;margin: 0px;clear: both;border-top: 1px solid black;background: white}
	.bottom td {font-size:9px;white-space:nowrap}
	.TabSelected {background-color: #000046;color: white;width: 95px;text-align: center;}
	.TabUnselected {background-color: #C3C3D3;width: 95px;text-align: center}
	.TabSpacer {width: 5px;}.TabLine {background-color: #000046;}
	.batch {text-align: center;margin-top: 8px;}
	.even {background-color: #E1E1E9;}
	.odd {}
	div#lightbox {padding:6px;border:1px solid blue;background:#e0e0e0;-moz-border-radius: 10px;-webkit-border-radius:10px;border-radius:10px;text-decoration:none}
	body div#overlay {
	z-index: 9998;
	background-color: #369;
	filter: alpha(opacity=30);
	-moz-opacity: 0.3;
	opacity: 0.3}'
! !

!SGLibrary methodsFor: 'other' stamp: 'obi 10/31/2008 13:20'!
magritteCss
	^'ul.errors {background:#f88;padding-left:16px;border:1px dotted #ccc}
	ul.errors li {margin:0;padding:2px;list-style:square}
	form th {padding-left:3px;padding-top:2px; background:#e0e0e0;font-weight:normal}
	form th.required {background:#ffffc0}'
! !

!SGLibrary methodsFor: 'other' stamp: 'obi 11/07/2008 13:52'!
notUsedCss
	^'	.page .contents td a {-moz-border-radius: 5px;-webkit-border-radius:5px;border-radius:5px;text-decoration:none;background:#888;border:1px solid #666;padding:0px 4px 0px 4px;color:white;white-space:nowrap}
	.page .contents td a:hover {background:#aaa}'
! !

!SGLibrary methodsFor: 'other' stamp: 'obi 10/31/2008 12:09'!
selectorsToInclude
	^ #(#adminCss #magritteCss)
! !

WAFileLibrary subclass: #SGProjectLibrary
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Scaffolding'!

!SGProjectLibrary methodsFor: 'other' stamp: 'obi 11/25/2008 14:10'!
defaultCss
	^'
	body {margin: 0px;padding: 10px;border: 0px}
	body, div, tr, td, th, form input, form submit {font-family: verdana, Arial, Helvetica, sans-serif;font-size: 11px;}
	table, tr, td, th, form {margin: 0px;padding: 0px;border: 0px;}
	th {text-align: left;}
	td {padding:1px 2px} 
	a, tr th a, tr th a:hover {background:none;color:#5588ff;text-decoration: none;border:0}
	.page .contents td {padding:2px 1px}
	td a, a.action {-moz-border-radius: 5px;-webkit-border-radius:5px;border-radius:5px;text-decoration:none;background:#888;border:1px solid #666;padding:0px 4px 0px 4px;color:white;white-space:nowrap}
	td a:hover, a.action:hover {background:#aaa}
	
	h1,h2,h3 {margin-top: 0px}
	h1 {margin-bottom: 2px;font-size: 24px;}
	h2 {margin-bottom: 1px;font-size: 15px;}
	h3 {margin-bottom: 1px;font-size: 13px;}
	div#lightbox {padding:6px;border:1px solid blue;background:#e0e0e0;-moz-border-radius: 10px;-webkit-border-radius:10px;border-radius:10px;text-decoration:none}
	.rounded {-moz-border-radius: 10px;-webkit-border-radius:10px;border-radius:10px}
	body div#overlay {z-index: 9998; background-color: #369; filter: alpha(opacity=30); -moz-opacity: 0.3; 	opacity: 0.3}
	.list th {font-size:150%}
	.list th a {text-decoration:underline}
	div.login {margin-top:60px; padding:10px;background:#e0e0e0; border: 1px solid #ccc; width:400px }'
! !

!SGProjectLibrary methodsFor: 'other' stamp: 'obi 11/05/2008 17:06'!
magritteCss
	^'ul.errors {background:#f88;padding-left:16px;border:1px dotted #ccc}
	ul.errors li {margin:0;padding:2px;list-style:square}
	form th {padding-left:3px;padding-top:2px; background:#e0e0e0;font-weight:normal}
	form th.required {background:#ffffc0}
	.even {background-color: #E1E1E9}'
! !

!SGProjectLibrary methodsFor: 'other' stamp: 'obi 11/04/2008 15:03'!
selectorsToInclude
	^ #(#defaultCss #magritteCss)
! !

MARelationDescription subclass: #SGTwowayRelationDescription
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Scaffolding'!

!SGTwowayRelationDescription class methodsFor: 'accessing' stamp: 'obi 05/12/2009 10:58'!
defaultComponentClasses
	^ Array with: SGTwowayRelationComponent! !

!SGTwowayRelationDescription class methodsFor: 'accessing' stamp: 'obi 04/21/2009 12:58'!
defaultDefault
	^OrderedCollection new
"	^SGTwowayRelation new"! !

!SGTwowayRelationDescription class methodsFor: 'accessing-description' stamp: 'obi 04/19/2009 10:57'!
descriptionClasses
	^ MAMultipleOptionDescription new
		accessor: #classes;
		label: 'Classes';
		priority: 400;
		options: [ SGModel allSubclasses ] asDynamicObject;
		reference: MAClassDescription new;
		yourself! !

!SGTwowayRelationDescription class methodsFor: 'accessing' stamp: 'obi 04/19/2009 11:04'!
isAbstract
	^ false! !

!SGTwowayRelationDescription class methodsFor: 'accessing' stamp: 'obi 04/19/2009 11:03'!
label
	^ 'Twoway Relation'! !

!SGTwowayRelationDescription methodsFor: 'accessing' stamp: 'obi 04/20/2009 11:40'!
isDefinitive
	^false! !

!SGTwowayRelationDescription methodsFor: 'accessing' stamp: 'obi 04/20/2009 11:37'!
isOrdered
	^true! !

!SGTwowayRelationDescription methodsFor: 'accessing' stamp: 'obi 04/20/2009 11:37'!
isSorted
	^false! !

!SGTwowayRelationDescription methodsFor: 'accessing' stamp: 'obi 05/12/2009 12:32'!
kind
	^ Collection! !

!SGTwowayRelationDescription methodsFor: 'accessors' stamp: 'obi 04/22/2009 13:40'!
selectorAccessor: aSelector
	"Use SGTwowayRelationAccessor instead of default one"

	self accessor: (SGTwowayRelationAccessor selector: aSelector)! !

!SGTwowayRelationDescription methodsFor: 'accessing' stamp: 'obi 04/22/2009 13:35'!
targetSelector
	^self accessor targetSelector! !

!SGTwowayRelationDescription methodsFor: 'updating' stamp: 'obi 04/21/2009 13:20'!
targetSelector: aSymbol
	"the accessor must be set before"
	self accessor targetSelector: aSymbol! !

!SGTwowayRelationDescription methodsFor: 'method-creation' stamp: 'obi 05/05/2009 13:00'!
updatingMethods
	| newLine sourceString sourceString2 var |
	newLine := Character lf asString.
	var := self accessor readSelector asString.
	sourceString := String new.
	sourceString add: var , 'Add: aModel', newLine , newLine.
	sourceString add: '	(self accessorFor: #' , var , ') write: aModel to: self'.
	sourceString2 := String new.
	sourceString2 add: var , 'Remove: aModel', newLine , newLine.
	sourceString2 add: '	(self accessorFor: #' , var , ') remove: aModel from: self'.
	^Array with: sourceString with: sourceString2! !

MASingleOptionDescription subclass: #SGSingleOptionDescription
	instanceVariableNames: 'displayStringSelector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Scaffolding'!

!SGSingleOptionDescription methodsFor: 'accessing' stamp: 'obi 12/29/2008 12:03'!
displayStringSelector

   "Return the value of the instance variable 'displayStringSelector'."
   ^displayStringSelector
! !

!SGSingleOptionDescription methodsFor: 'updating' stamp: 'obi 12/29/2008 12:03'!
displayStringSelector: newValue

   "Modify the value of the instance variable 'displayStringSelector'."
   displayStringSelector := newValue
! !

!SGSingleOptionDescription methodsFor: 'initialization' stamp: 'obi 12/29/2008 12:07'!
initialize
	super initialize.
	displayStringSelector := #displayString! !

!SGSingleOptionDescription methodsFor: 'private' stamp: 'obi 12/29/2008 12:07'!
labelForOption: anObject
	^anObject perform: displayStringSelector! !

!SGSingleOptionDescription methodsFor: 'accessing' stamp: 'obi 12/29/2008 12:05'!
options
	^ options isNil
		ifTrue: [#()]
		ifFalse: [options value]! !

WAComponent subclass: #SGComponent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Scaffolding'!

!SGComponent methodsFor: 'convenience' stamp: 'obi 11/08/2008 10:03'!
confirm: aString
	"use lightbox instead of the default call:"
	^self lightbox: (WAYesOrNoDialog new
						addMessage: aString;
						yourself)! !

!SGComponent methodsFor: 'convenience' stamp: 'obi 11/25/2008 14:12'!
inform: aString
	"use lightbox instead of the default call:"
	^self lightbox: (WAFormDialog new
		addMessage: aString;
		yourself)! !

!SGComponent methodsFor: 'convenience' stamp: 'DataCurator 11/10/2008 12:04'!
title

	^'Scaffolding for GemStone/S'.
! !

!SGComponent methodsFor: 'convenience' stamp: 'DataCurator 11/10/2008 12:05'!
updateRoot: anHtmlRoot

	super updateRoot: anHtmlRoot.
	anHtmlRoot title: self title.
! !

SGComponent subclass: #SGFormComponent
	instanceVariableNames: 'model editor parent'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Scaffolding'!

SGFormComponent subclass: #SGEditorComponent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Scaffolding'!

!SGEditorComponent class methodsFor: 'instance creation' stamp: 'obi 11/07/2008 14:33'!
on: aModel
	^self new model: aModel; yourself! !

!SGEditorComponent methodsFor: 'private' stamp: 'obi 11/07/2008 14:30'!
buildEditor
	^model open! !

!SGEditorComponent methodsFor: 'rendering' stamp: 'obi 11/08/2008 01:12'!
renderTitleOn: html
	html heading: 'Edit: ', model displayString! !

!SGFormComponent class methodsFor: 'instance creation' stamp: 'obi 11/08/2008 01:05'!
on: aModel
	^self new model: aModel; yourself! !

!SGFormComponent methodsFor: 'private' stamp: 'obi 11/08/2008 01:08'!
buildEditor
	^self subclassResponsibility! !

!SGFormComponent methodsFor: 'accessing' stamp: 'obi 11/08/2008 01:05'!
children
	^Array with: editor! !

!SGFormComponent methodsFor: 'accessing' stamp: 'obi 11/08/2008 01:05'!
editor
	editor ifNil: [editor := self buildEditor].
	^editor! !

!SGFormComponent methodsFor: 'accessing' stamp: 'obi 11/08/2008 01:05'!
model: aModel
	model := aModel! !

!SGFormComponent methodsFor: 'accessing' stamp: 'obi 11/08/2008 06:37'!
parent
	^parent
	! !

!SGFormComponent methodsFor: 'accessing' stamp: 'obi 11/08/2008 06:37'!
parent: aComponent
	parent := aComponent
	! !

!SGFormComponent methodsFor: 'rendering' stamp: 'obi 11/08/2008 01:05'!
renderActionsOn: html
	! !

!SGFormComponent methodsFor: 'rendering' stamp: 'obi 11/08/2008 01:05'!
renderBodyOn: html
	html render: self editor! !

!SGFormComponent methodsFor: 'rendering' stamp: 'obi 11/08/2008 01:05'!
renderContentOn: html
	self renderHeaderOn: html.
	self renderBodyOn: html.
	self renderActionsOn: html.
	self renderFooterOn: html.! !

!SGFormComponent methodsFor: 'rendering' stamp: 'obi 11/08/2008 01:05'!
renderFooterOn: html
	! !

!SGFormComponent methodsFor: 'rendering' stamp: 'obi 11/08/2008 01:11'!
renderHeaderOn: html
	html table: [
			html tableRow: [
				html tableData: [
					html anchor
						class: 'back';
						callback: [self answer];
						with: '<'].
				html tableData: [self renderTitleOn: html]]]! !

!SGFormComponent methodsFor: 'rendering' stamp: 'obi 11/08/2008 01:11'!
renderTitleOn: html
	html heading: model displayString! !

SGFormComponent subclass: #SGViewComponent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Scaffolding'!

!SGViewComponent class methodsFor: 'instance creation' stamp: 'obi 11/08/2008 01:07'!
on: aModel
	^self new model: aModel; yourself! !

!SGViewComponent methodsFor: 'private' stamp: 'obi 11/08/2008 01:08'!
buildEditor
	^model show! !

!SGViewComponent methodsFor: 'rendering' stamp: 'obi 11/08/2008 01:12'!
renderTitleOn: html
	html heading: 'Show: ', model displayString! !

SGComponent subclass: #SGListComponent
	instanceVariableNames: 'table parent'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Scaffolding'!

!SGListComponent class methodsFor: 'accessing' stamp: 'obi 11/08/2008 04:36'!
modelClass
	^self subclassResponsibility! !

!SGListComponent methodsFor: 'private' stamp: 'obi 11/08/2008 06:29'!
buildTable
	^self modelClass projectReportOn: self list! !

!SGListComponent methodsFor: 'accessing' stamp: 'obi 11/08/2008 04:47'!
children
	^Array with: table! !

!SGListComponent methodsFor: 'initialize-release' stamp: 'obi 11/08/2008 04:48'!
initialize
	super initialize.
	table := self buildTable.
	self onAnswer: [:result | self refresh].! !

!SGListComponent methodsFor: 'private' stamp: 'obi 11/08/2008 06:28'!
list
	^self modelClass allInstancesSorted! !

!SGListComponent methodsFor: 'accessing' stamp: 'obi 11/08/2008 04:48'!
modelClass
	^self class modelClass! !

!SGListComponent methodsFor: 'accessing' stamp: 'obi 11/08/2008 06:36'!
parent
	^parent
	! !

!SGListComponent methodsFor: 'accessing' stamp: 'obi 11/08/2008 06:37'!
parent: aComponent
	parent := aComponent
	! !

!SGListComponent methodsFor: 'initialize-release' stamp: 'obi 11/08/2008 04:48'!
refresh
	table rows: self modelClass allInstances
! !

!SGListComponent methodsFor: 'rendering' stamp: 'obi 11/08/2008 04:50'!
renderContentOn: html
	self renderTableOn: html.
! !

!SGListComponent methodsFor: 'rendering-convenience' stamp: 'obi 11/08/2008 06:17'!
renderListActionsOn: html model: aModel
	! !

!SGListComponent methodsFor: 'rendering-convenience' stamp: 'obi 11/08/2008 06:04'!
renderListHeaderActionsOn: html model: aModel
	! !

!SGListComponent methodsFor: 'rendering-convenience' stamp: 'obi 11/08/2008 06:27'!
renderListOn: html
	| visibleReadAccesors otherSelectors headSelector |
	visibleReadAccesors := self modelClass allVisibleReadAccessors.
	headSelector := visibleReadAccesors first.
	otherSelectors := visibleReadAccesors copyFrom: 2 to: visibleReadAccesors size.
	html table
		class: 'list'; 
		with: [
		self list 
			do: [:model |
					html tableRow: 
							[html tableHeading
								with: [self renderListTitleOn: html model: model method: headSelector].
							html tableData: [
									self renderListHeaderActionsOn: html model: model  ]].
					 otherSelectors do: [:aSelector |
						html tableRow: 
							[html tableData colSpan: 99; with: (model perform: aSelector) ] ].
					self renderPrivateRowsOn: html model: model.
					html tableRow: 
							[html tableData colSpan: 99; with: [self renderListActionsOn: html model: model] ] ]	 ]! !

!SGListComponent methodsFor: 'rendering-convenience' stamp: 'obi 11/08/2008 06:27'!
renderListTitleOn: html model: aModel method: headSelector
	html anchor callback: [WARenderLoop new call: (aModel perform: self modelClass defaultShowAction)]; with: (aModel perform: headSelector)! !

!SGListComponent methodsFor: 'rendering-convenience' stamp: 'obi 11/08/2008 06:23'!
renderPrivateRowsOn: html model: aModel
	! !

!SGListComponent methodsFor: 'rendering' stamp: 'obi 11/08/2008 04:50'!
renderTableOn: html
	html render: table! !

!SGListComponent methodsFor: 'private' stamp: 'obi 11/08/2008 05:49'!
rows: rows
	table isNil ifFalse: [table rows: rows]! !

!SGListComponent methodsFor: 'accessing' stamp: 'obi 11/08/2008 04:48'!
table
	^table! !

SGComponent subclass: #SGProjectComponent
	instanceVariableNames: 'table'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Scaffolding'!

!SGProjectComponent class methodsFor: 'accessing' stamp: 'obi 11/04/2008 14:27'!
listActions
	^self modelClass defaultListActions! !

!SGProjectComponent class methodsFor: 'accessing' stamp: 'DataCurator 11/08/2008 00:36'!
modelClass
	^SGProject! !

!SGProjectComponent methodsFor: 'private' stamp: 'obi 11/08/2008 10:40'!
buildTable
	^self modelClass listClass new parent: self; yourself! !

!SGProjectComponent methodsFor: 'private' stamp: 'obi 11/08/2008 05:12'!
buildTable2
	^self modelClass projectReport! !

!SGProjectComponent methodsFor: 'accessing' stamp: 'obi 11/04/2008 14:52'!
children
	^Array with: table! !

!SGProjectComponent methodsFor: 'initialize-release' stamp: 'obi 11/05/2008 09:48'!
initialize
	super initialize.
	table := self buildTable.
	self onAnswer: [:result | self refresh].! !

!SGProjectComponent methodsFor: 'accessing' stamp: 'obi 11/04/2008 14:29'!
modelClass
	^self class modelClass! !

!SGProjectComponent methodsFor: 'private' stamp: 'obi 11/05/2008 10:02'!
refresh
	table rows: self modelClass allInstances
! !

!SGProjectComponent methodsFor: 'rendering' stamp: 'obi 11/05/2008 09:50'!
renderActionsOn: html
	html break.
	self class listActions
		do: [:anAction |
				html anchor
					class: 'action';
					callback: [
							self call: (self modelClass perform: anAction key).
							self refresh];
					with: anAction value]
		separatedBy: [html space]
	! !

!SGProjectComponent methodsFor: 'rendering' stamp: 'obi 11/04/2008 14:32'!
renderBodyOn: html
	self renderTableOn: html! !

!SGProjectComponent methodsFor: 'rendering' stamp: 'obi 11/04/2008 14:18'!
renderContentOn: html
	self renderIndexOn: html.
	self renderBodyOn: html.
	self renderActionsOn: html.! !

!SGProjectComponent methodsFor: 'rendering' stamp: 'obi 11/04/2008 14:31'!
renderIndexOn: html
	self renderTitleOn: html! !

!SGProjectComponent methodsFor: 'rendering' stamp: 'obi 11/04/2008 14:35'!
renderTableOn: html
	html render: table! !

!SGProjectComponent methodsFor: 'rendering' stamp: 'obi 11/04/2008 14:31'!
renderTitleOn: html
	html heading: self modelClass displayString! !

!SGProjectComponent methodsFor: 'accessing' stamp: 'obi 11/04/2008 14:32'!
table
	^table! !

!MAContainer methodsFor: '*seaside-scaffolding-query' stamp: 'obi 05/12/2009 12:37'!
fieldDescriptions
	 ^self children select: [:desc | desc kind ~~ Collection]
! !

!MAContainer methodsFor: '*seaside-scaffolding-query' stamp: 'obi 05/12/2009 12:37'!
noneRelationDescriptions
	 ^self fieldDescriptions
! !

!MAContainer methodsFor: '*seaside-scaffolding-query' stamp: 'obi 05/12/2009 12:36'!
relationDescriptions
	 ^self children select: [:desc | desc kind ~~ Collection]
! !

SequenceableCollection subclass: #SGTwowayRelation
	instanceVariableNames: 'model'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Scaffolding'!

!SGTwowayRelation methodsFor: 'Adding' stamp: 'obi 04/14/2009 11:51'!
add: aModel withAttribute: aSymbol
	self add: (aModel perform: aSymbol)
	! !

!SGTwowayRelation methodsFor: 'printing' stamp: 'obi 04/20/2009 11:50'!
asString
	| result |
	result := String new.
	self targets do: [:model | result add: model asString] separatedBy: [result add: ', '].
	^result! !

!SGTwowayRelation methodsFor: 'Enumerating' stamp: 'obi 04/14/2009 12:25'!
elf size do: [ :i | (aBlock value: (self at: i) model) ifTrue: [^(self at: i) model]]. 
	^exceptionBlock value
! !

!SGTwowayRelation methodsFor: 'accessing' stamp: 'obi 04/14/2009 12:14'!
firstTarget
	self size == 0 ifTrue: [^nil].
	^(self at: 1) model! !

!SGTwowayRelation methodsFor: 'Searching' stamp: 'obi 04/14/2009 12:23'!
includesTarget: anObject
	1 to: self size do: [:i |
		  (anObject == (self at: i) model) ifTrue: [^true] ].

	^false
! !

!SGTwowayRelation methodsFor: 'accessing' stamp: 'obi 04/14/2009 11:48'!
model

   "Return the value of the instance variable 'model'."
   ^model
! !

!SGTwowayRelation methodsFor: 'updating' stamp: 'obi 04/14/2009 11:48'!
model: newValue

   "Modify the value of the instance variable 'model'."
   model := newValue
! !

!SGTwowayRelation methodsFor: 'printing' stamp: 'obi 04/20/2009 11:46'!
printOn: aStream
	self targets printOn: aStream! !

!SGTwowayRelation methodsFor: 'Removing' stamp: 'obi 04/14/2009 11:54'!
remove: aModel withAttribute: aSymbol
	self remove: aModel withAttribute: aSymbol ifAbsent: [self _errorNotFound: aModel]
	! !

!SGTwowayRelation methodsFor: 'Removing' stamp: 'obi 04/14/2009 11:53'!
remove: aModel withAttribute: aSymbol ifAbsent: aBlock
	self remove: (aModel perform: aSymbol) ifAbsent: aBlock
	! !

!SGTwowayRelation methodsFor: 'Enumerating' stamp: 'obi 04/14/2009 12:25'!
self size do: [ :i | (aBlock value: (self at: i) model) ifTrue: [^(self at: i) model]]. 
	^exceptionBlock value
! !

!SGTwowayRelation methodsFor: 'accessing' stamp: 'obi 04/17/2009 12:51'!
targets
	| result |
	result := OrderedCollection new.
	self targetsDo: [:model | result add: model].
	^result! !

!SGTwowayRelation methodsFor: 'Enumerating' stamp: 'obi 04/14/2009 12:26'!
targetsDetect: aBlock
	^self targetsDetect: aBlock ifNone: [].
! !

!SGTwowayRelation methodsFor: 'Enumerating' stamp: 'obi 04/14/2009 12:25'!
targetsDetect: aBlock ifNone: exceptionBlock

	1 to: self size do: [ :i | (aBlock value: (self at: i) model) ifTrue: [^(self at: i) model]]. 
	^exceptionBlock value
! !

!SGTwowayRelation methodsFor: 'Enumerating' stamp: 'obi 04/14/2009 12:19'!
targetsDo: aBlock
	1 to: self size do: [:idx | aBlock value: (self at: idx) model]! !

WASession subclass: #SGSession
	instanceVariableNames: 'mandator user project'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Scaffolding'!

!SGSession methodsFor: 'initialize-release' stamp: 'obi 11/04/2008 07:49'!
initialize
	super initialize.
	mandator := SGMandator default! !

!SGSession methodsFor: 'accessing' stamp: 'obi 11/04/2008 07:48'!
mandator
	^mandator! !

!SGSession methodsFor: 'accessing' stamp: 'obi 11/04/2008 07:48'!
mandator: aMandator
	mandator := aMandator! !

!SGSession methodsFor: 'accessing' stamp: 'obi 11/25/2008 13:29'!
project
	^project! !

!SGSession methodsFor: 'accessing' stamp: 'obi 11/25/2008 13:29'!
project: aMandator
	project := aMandator! !

!SGSession methodsFor: 'accessing' stamp: 'obi 11/25/2008 13:26'!
user
	^user! !

!SGSession methodsFor: 'accessing' stamp: 'obi 11/25/2008 13:26'!
user: aUser
	user := aUser! !

!SGSession methodsFor: 'accessing' stamp: 'obi 11/25/2008 14:17'!
userName
	^user isNil ifTrue: [''] ifFalse: [user loginName]! !

!SGSession methodsFor: 'accessing' stamp: 'obi 11/25/2008 14:17'!
userName: aName
	user := SGUser detect: [:each | each loginName = aName].
	^user! !

Object subclass: #SGModel
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Scaffolding-Model'!

SGModel class
	instanceVariableNames: 'Meta'!

!SGModel commentStamp: '<historical>' prior: 0!
Root class for all my model classes.
!

SGModel class
	instanceVariableNames: 'Meta'!

SGModel subclass: #SGGroup
	instanceVariableNames: 'name label users defaultMandator allowedMandators accessRights'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Scaffolding-Model'!

!SGGroup commentStamp: '<historical>' prior: 0!
group!

!SGGroup class methodsFor: 'accessing-description' stamp: 'obi 05/05/2009 13:14'!
descriptionAccessRights
	^MAStringDescription new
		selectorAccessor: 'accessRights';
		cssClass: 'string';
		priority: 1;
		label: 'AccessRights';
		yourself! !

!SGGroup class methodsFor: 'accessing-description' stamp: 'obi 05/05/2009 13:14'!
descriptionAllowedMandators
	^MAStringDescription new
		selectorAccessor: 'allowedMandators';
		cssClass: 'string';
		priority: 1;
		label: 'AllowedMandators';
		yourself! !

!SGGroup class methodsFor: 'accessing-description' stamp: 'obi 05/05/2009 13:14'!
descriptionDefaultMandator
	^MAStringDescription new
		selectorAccessor: 'defaultMandator';
		cssClass: 'string';
		priority: 1;
		label: 'DefaultMandator';
		yourself! !

!SGGroup class methodsFor: 'accessing-description' stamp: 'obi 05/05/2009 13:14'!
descriptionLabel
	^MAStringDescription new
		selectorAccessor: 'label';
		cssClass: 'string';
		priority: 1;
		label: 'Label';
		yourself! !

!SGGroup class methodsFor: 'accessing-description' stamp: 'obi 05/05/2009 13:14'!
descriptionName
	^MAStringDescription new
		selectorAccessor: 'name';
		cssClass: 'string';
		priority: 1;
		label: 'Name';
		yourself! !

!SGGroup class methodsFor: 'accessing-description' stamp: 'obi 6/13/2010 19:54'!
descriptionUsers
	^SGTwowayRelationDescription new
		selectorAccessor: 'users';
		targetSelector: #groups;
		cssClass: 'object';
		priority: 100;
		label: 'Users';
		visible: true;
		classes: (OrderedCollection with: SGUser);
		yourself! !

!SGGroup class methodsFor: 'testing' stamp: 'obi 11/01/2008 11:07'!
isBasic
	^name == #SGGroup! !

!SGGroup methodsFor: 'Accessing' stamp: 'obi 10/31/2008 11:44'!
accessRights

   "Return the value of the instance variable 'accessRights'."
   ^accessRights
! !

!SGGroup methodsFor: 'Updating' stamp: 'obi 10/31/2008 11:44'!
accessRights: newValue

   "Modify the value of the instance variable 'accessRights'."
   accessRights := newValue
! !

!SGGroup methodsFor: 'Accessing' stamp: 'obi 10/31/2008 11:44'!
allowedMandators

   "Return the value of the instance variable 'allowedMandators'."
   ^allowedMandators
! !

!SGGroup methodsFor: 'Updating' stamp: 'obi 10/31/2008 11:44'!
allowedMandators: newValue

   "Modify the value of the instance variable 'allowedMandators'."
   allowedMandators := newValue
! !

!SGGroup methodsFor: 'Accessing' stamp: 'obi 10/31/2008 11:44'!
defaultMandator

   "Return the value of the instance variable 'defaultMandator'."
   ^defaultMandator
! !

!SGGroup methodsFor: 'Updating' stamp: 'obi 10/31/2008 11:44'!
defaultMandator: newValue

   "Modify the value of the instance variable 'defaultMandator'."
   defaultMandator := newValue
! !

!SGGroup methodsFor: 'Accessing' stamp: 'obi 10/31/2008 11:44'!
label

   "Return the value of the instance variable 'label'."
   ^label
! !

!SGGroup methodsFor: 'Updating' stamp: 'obi 10/31/2008 11:44'!
label: newValue

   "Modify the value of the instance variable 'label'."
   label := newValue
! !

!SGGroup methodsFor: 'Accessing' stamp: 'obi 10/31/2008 11:44'!
name

   "Return the value of the instance variable 'name'."
   ^name
! !

!SGGroup methodsFor: 'Updating' stamp: 'obi 10/31/2008 11:44'!
name: newValue

   "Modify the value of the instance variable 'name'."
   name := newValue
! !

!SGGroup methodsFor: 'Accessing' stamp: 'obi 10/31/2008 11:44'!
users

   "Return the value of the instance variable 'users'."
   ^users
! !

!SGGroup methodsFor: 'Updating' stamp: 'obi 10/31/2008 11:44'!
users: newValue

   "Modify the value of the instance variable 'users'."
   users := newValue
! !

!SGGroup methodsFor: 'updating' stamp: 'obi 05/05/2009 13:14'!
usersAdd: aModel

	(self accessorFor: #users) write: aModel to: self! !

!SGGroup methodsFor: 'updating' stamp: 'obi 05/05/2009 13:14'!
usersRemove: aModel

	(self accessorFor: #users) remove: aModel from: self! !

SGModel subclass: #SGMandator
	instanceVariableNames: 'name description classes properties accessRights users'
	classVariableNames: 'List'
	poolDictionaries: ''
	category: 'Seaside-Scaffolding-Model'!

!SGMandator class methodsFor: 'updating' stamp: 'obi 10/26/2008 00:16'!
add: aMandator
	self list add: aMandator! !

!SGMandator class methodsFor: 'public' stamp: 'obi 10/26/2008 11:10'!
allDo: aBlock
	self list do: aBlock! !

!SGMandator class methodsFor: 'accessing' stamp: 'obi 10/31/2008 09:17'!
allInstances
	^self list! !

!SGMandator class methodsFor: 'testing' stamp: 'obi 11/02/2008 09:40'!
canCreateSubclass
	^false! !

!SGMandator class methodsFor: 'testing' stamp: 'obi 11/02/2008 11:17'!
canEdit
	^false! !

!SGMandator class methodsFor: 'accessing' stamp: 'dkh 04/08/2009 11:05'!
default
	| aList newMandator |
	aList := self list.
	aList isEmpty
		ifTrue: [newMandator := self new.
				newMandator name: 'Default'.
				newMandator description: 'Default mandator'.
				self add: newMandator].
	^self list anyOne! !

!SGMandator class methodsFor: 'descriptions' stamp: 'obi 10/31/2008 11:25'!
descriptionAccessRights
	^ MAToManyRelationDescription new
		classes: (Array new);
		accessor: #accessRights;
		label: 'Access Rights';
		default: Array new;
		priority: 100;
		visible: false;
		beDefinitive;
		beOrdered;
		yourself! !

!SGMandator class methodsFor: 'descriptions' stamp: 'obi 10/31/2008 11:30'!
descriptionClasses
	| desc |
	desc := self stringDescriptionOn: #classes.
	desc visible: false.
	^desc! !

!SGMandator class methodsFor: 'descriptions' stamp: 'obi 10/31/2008 11:35'!
descriptionName
	| desc |
	desc := self stringDescriptionOn: #name.
	desc required: true.
	^desc! !

!SGMandator class methodsFor: 'descriptions' stamp: 'obi 10/31/2008 11:29'!
descriptionProperties
	| desc |
	desc := self stringDescriptionOn: #properties.
	desc visible: false.
	^desc! !

!SGMandator class methodsFor: 'descriptions' stamp: 'obi 10/31/2008 15:30'!
descriptionUsers
	^ MAToManyRelationDescription new
		classes: (Array with: SGUser);
		accessor: #users;
		default: Array new;
		label: 'Users';
		priority: 100;
		beDefinitive;
		beOrdered;
		yourself! !

!SGMandator class methodsFor: 'testing' stamp: 'obi 11/02/2008 11:18'!
isBasic
	^true! !

!SGMandator class methodsFor: 'accessing' stamp: 'obi 04/07/2009 13:29'!
list
	List isNil ifTrue: [List := IdentitySet new].
	^List! !

!SGMandator class methodsFor: 'updating' stamp: 'obi 10/26/2008 00:34'!
remove: aMandator
	self list remove: aMandator ifAbsent: []! !

!SGMandator methodsFor: 'Accessing' stamp: 'obi 10/26/2008 00:29'!
accessRights

   "Return the value of the instance variable 'accessRights'."
   ^accessRights
! !

!SGMandator methodsFor: 'Updating' stamp: 'obi 10/26/2008 00:29'!
accessRights: newValue

   "Modify the value of the instance variable 'accessRights'."
   accessRights := newValue
! !

!SGMandator methodsFor: 'instance-accessing' stamp: 'obi 10/26/2008 00:58'!
add: anObject 
	^self add: anObject to: anObject class
! !

!SGMandator methodsFor: 'instance-accessing' stamp: 'obi 10/26/2008 00:46'!
add: anObject to: aClass
	(self instancesAt: aClass name) add: anObject.
	^anObject! !

!SGMandator methodsFor: 'instance-accessing' stamp: 'obi 10/26/2008 00:47'!
addAll: aColl to: aClass
	(self instancesAt: aClass name) addAll: aColl.
	^aColl! !

!SGMandator methodsFor: 'public' stamp: 'obi 12/18/2008 11:37'!
and1: anObject

	self & anObject

! !

!SGMandator methodsFor: 'public' stamp: 'obi 10/26/2008 10:27'!
beCurrent
	SGSystem currentMandator: self! !

!SGMandator methodsFor: 'Accessing' stamp: 'obi 10/26/2008 00:29'!
classes

   "Return the value of the instance variable 'classes'."
   ^classes
! !

!SGMandator methodsFor: 'Updating' stamp: 'obi 10/26/2008 00:29'!
classes: newValue

   "Modify the value of the instance variable 'classes'."
   classes := newValue
! !

!SGMandator methodsFor: 'instance-accessing' stamp: 'obi 10/26/2008 10:23'!
classesDo: aBlock

	classes keys do: aBlock
! !

!SGMandator methodsFor: 'Accessing' stamp: 'obi 10/26/2008 00:29'!
description

   "Return the value of the instance variable 'description'."
   ^description
! !

!SGMandator methodsFor: 'Updating' stamp: 'obi 10/26/2008 00:29'!
description: newValue

   "Modify the value of the instance variable 'description'."
   description := newValue
! !

!SGMandator methodsFor: 'instance-accessing' stamp: 'obi 10/26/2008 11:09'!
hasInstancesOf: className
	^(classes at: className ifAbsent: [#()]) notEmpty! !

!SGMandator methodsFor: 'initialize' stamp: 'obi 6/13/2010 16:49'!
initialize
	super initialize.
	name := 'No name'.
	description := ''.
	classes := Dictionary new.
	properties := Dictionary new.! !

!SGMandator methodsFor: 'instance-accessing' stamp: 'obi 10/26/2008 00:47'!
instancesAt: className
	^classes at: className ifAbsent: [classes at: className put: self newInstanceCollection]! !

!SGMandator methodsFor: 'instance-accessing' stamp: 'obi 10/26/2008 10:23'!
instancesDo: aBlock

	classes values do: aBlock
! !

!SGMandator methodsFor: 'Accessing' stamp: 'obi 10/26/2008 00:29'!
name

   "Return the value of the instance variable 'name'."
   ^name
! !

!SGMandator methodsFor: 'Updating' stamp: 'obi 10/26/2008 00:29'!
name: newValue

   "Modify the value of the instance variable 'name'."
   name := newValue
! !

!SGMandator methodsFor: 'private' stamp: 'obi 10/26/2008 00:49'!
newInstanceCollection
	"maybe we could also use RcIdentityBag"
	^IdentityBag new! !

!SGMandator methodsFor: 'Accessing' stamp: 'obi 10/26/2008 00:29'!
properties

   "Return the value of the instance variable 'properties'."
   ^properties
! !

!SGMandator methodsFor: 'Updating' stamp: 'obi 10/26/2008 00:29'!
properties: newValue

   "Modify the value of the instance variable 'properties'."
   properties := newValue
! !

!SGMandator methodsFor: 'instance-accessing' stamp: 'obi 10/26/2008 00:57'!
remove: anObject
	^self remove: anObject from: anObject class
! !

!SGMandator methodsFor: 'instance-accessing' stamp: 'obi 10/26/2008 00:58'!
remove: anObject from: aClass
	^(self instancesAt: aClass name) remove: anObject
! !

!SGMandator methodsFor: 'instance-accessing' stamp: 'obi 10/26/2008 11:14'!
removeClassNamed: className
	^classes removeKey: className ifAbsent: []! !

!SGMandator methodsFor: 'Accessing' stamp: 'obi 10/31/2008 09:44'!
users

   "Return the value of the instance variable 'users'."
   ^users
! !

!SGMandator methodsFor: 'Updating' stamp: 'obi 10/31/2008 09:44'!
users: newValue

   "Modify the value of the instance variable 'users'."
   users := newValue
! !

!SGModel class methodsFor: 'accessing' stamp: 'obi 11/01/2008 10:33'!
<= aClass
	^name <= aClass name! !

!SGModel class methodsFor: 'attributes-accessing' stamp: 'obi 05/05/2009 12:54'!
accessorFor: anAttribute
	self allAttributeDescriptions do: [:desc | | acc | (acc := desc accessor) readSelector == anAttribute ifTrue: [^acc]].
	^nil! !

!SGModel class methodsFor: 'attributes-accessing' stamp: 'obi 05/16/2009 10:51'!
accessorForAttribute: anAttribute
	self attributeContainer children do: [:desc | | accessor | accessor := desc accessor. accessor readSelector == anAttribute ifTrue: [^accessor]].
	^nil! !

!SGModel class methodsFor: 'adding-removing' stamp: 'obi 12/20/2008 12:35'!
add: anObject
	^self currentMandator add: anObject! !

!SGModel class methodsFor: 'methods-creation' stamp: 'obi 12/17/2008 11:02'!
addClassesAccessorFor: aDescription on: sourceString
	| theClasses |
	theClasses := aDescription classes.
	theClasses isEmpty ifFalse: [
			| str |
			str := String new.
			str add: '		classes: (Array new add: '.
			theClasses do: [:cl | str add: cl name asString] separatedBy: [str add: '; add: '].
			str add: '; yourself);'.
			sourceString add: str]! !

!SGModel class methodsFor: 'attributes-accessing' stamp: 'obi 10/28/2008 12:56'!
addDescription: anAttributeDescription
	^self attributeContainer add: anAttributeDescription! !

!SGModel class methodsFor: 'attributes-accessing' stamp: 'obi 6/13/2010 19:40'!
allAttributeDescriptions
	| allDescriptions |

	allDescriptions := OrderedCollection new.
	self mySuperClasses reverseDo: [:cl |
		allDescriptions addAll: cl attributeContainer children].
	allDescriptions addAll: self attributeContainer children.
	^allDescriptions! !

!SGModel class methodsFor: 'accessing' stamp: 'obi 11/25/2008 13:18'!
allClasses
	^self withAllSubclasses sortAscending: #(#name)! !

!SGModel class methodsFor: 'accessing' stamp: 'obi 10/26/2008 00:50'!
allInstances
	^self currentMandator instancesAt: name! !

!SGModel class methodsFor: 'accessing' stamp: 'obi 10/31/2008 15:08'!
allInstancesSorted
	^self allInstances sortAscending: self keyAttribute! !

!SGModel class methodsFor: 'accessing' stamp: 'obi 01/06/2009 06:02'!
allUserClasses
	^(self withAllSubclasses select: [:cl | cl isBasic not]) sortAscending: #(#name)! !

!SGModel class methodsFor: 'attributes-accessing' stamp: 'obi 11/08/2008 05:28'!
allVisibleAttributeDescriptions
	^self allAttributeDescriptions select: [:each | each isVisible]! !

!SGModel class methodsFor: 'attributes-accessing' stamp: 'obi 11/08/2008 05:52'!
allVisibleReadAccessors
	^self allVisibleAttributeDescriptions collect: [:desc | desc accessor readSelector]! !

!SGModel class methodsFor: 'accessing' stamp: 'obi 04/21/2009 11:22'!
any
	self do: [:each | ^each].
	^nil! !

!SGModel class methodsFor: 'private' stamp: 'obi 6/13/2010 17:50'!
asLabel: aSelector
	^aSelector asString capitalized
	
	! !

!SGModel class methodsFor: 'accessing' stamp: 'obi 10/28/2008 12:42'!
attributeContainer
	^self metaAtKey: #attributeContainer ifAbsent: [self metaAtKey: #attributeContainer put: self descriptionContainer].! !

!SGModel class methodsFor: 'accessing' stamp: 'obi 10/28/2008 12:55'!
attributeDescriptions
	| theDescriptions |
	theDescriptions := self attributeContainer children.
	theDescriptions isEmpty ifTrue: [theDescriptions := self createDefaultDescriptions].
	^theDescriptions! !

!SGModel class methodsFor: 'attributes-accessing' stamp: 'obi 11/01/2008 09:55'!
attributesAsString
	| result |
	result := String new.
	self instVarNames do: [:aSymbol | result add: aSymbol asString] separatedBy: [result add: $;].
	^result! !

!SGModel class methodsFor: 'attributes-accessing' stamp: 'obi 12/17/2008 11:31'!
cacheDescriptionMethods
	"use this to cache manually defined or changed descriptions methods"
	| currentDescriptions theInstVars |
	currentDescriptions := self attributeDescriptions.
	theInstVars := self instVarNames.
	theInstVars do: [:aSymbol | | currentDescription |
		currentDescription := currentDescriptions detect: [:desc | desc accessor readSelector == aSymbol] ifNone: [].
		currentDescription isNil
			ifFalse: [| oldIndex newDescription |
					newDescription := self defaultDescriptionOn: aSymbol.
					newDescription isNil ifFalse: [
						oldIndex := self attributeContainer children indexOf: currentDescription.
						self descriptionAt: oldIndex put: newDescription ] ]
			ifTrue: [self addDescription: (self defaultDescriptionOn: aSymbol)]].
	^self attributeDescriptions
! !

!SGModel class methodsFor: 'testing' stamp: 'obi 11/02/2008 09:40'!
canCreateSubclass
	^true! !

!SGModel class methodsFor: 'testing' stamp: 'obi 11/01/2008 11:25'!
canDelete
	self subclasses isEmpty ifFalse: [ ^false ].
	(SGSystem hasInstancesOf: name) ifTrue: [^false].
	^true! !

!SGModel class methodsFor: 'testing' stamp: 'obi 11/02/2008 11:18'!
canEdit
	^self isBasic not! !

!SGModel class methodsFor: 'attributes-accessing' stamp: 'obi 11/07/2008 06:47'!
changeDescription: anAttributeDescription to: newAttributeDescription
	self removeDescription: anAttributeDescription.
	^self addDescription: newAttributeDescription! !

!SGModel class methodsFor: 'querying' stamp: 'obi 10/26/2008 01:02'!
collect: aBlock
	^self allInstances collect: aBlock! !

!SGModel class methodsFor: 'meta-accessing' stamp: 'obi 6/13/2010 17:19'!
comment: aText
	! !

!SGModel class methodsFor: 'methods-creation' stamp: 'obi 6/13/2010 18:08'!
compile: sourceString classified: aSymbol
	^SGSystem compile: sourceString intoClass: self classified: aSymbol! !

!SGModel class methodsFor: 'methods-creation' stamp: 'obi 6/13/2010 19:48'!
compileAccessingAndUpdatingMethodsForDescription: aDescription
	| sourceStringArray | 
	sourceStringArray := aDescription accessingMethods.
	sourceStringArray do: [:sourceString |
			self 
				compile: sourceString
				classified: #'accessing'].
	sourceStringArray := aDescription updatingMethods.
	sourceStringArray do: [:sourceString |
				self 
					compile: sourceString
					classified: #'updating']
	
! !

!SGModel class methodsFor: 'methods-creation' stamp: 'obi 6/13/2010 19:53'!
compileAccessingMethodsFor: anArrayOfSymbols

	| newLine accessing updating allVarNames varName |
	accessing := #accessing.
	updating := #updating.
	varName := 'newValue'.
	allVarNames := self allInstVarNames.
	[allVarNames includes: varName] whileTrue: [
	  varName := 'z' , varName ].
	newLine:= Character lf asString.
	anArrayOfSymbols do:
	    [ :var |
	      (self compile: (var , newLine , newLine ,
            '   "Return the value of the instance variable ''' , var ,
            '''."' , newLine , '   ^' , var , newLine)
            classified: accessing).

      (self compile: (var , ': ' , varName , newLine , newLine ,
            '   "Modify the value of the instance variable ''' , var ,
            '''."' , newLine , '   ' , var , ' := ' , varName , newLine)
            classified: updating) ]
! !

!SGModel class methodsFor: 'methods-creation' stamp: 'obi 6/13/2010 19:50'!
compileDescriptionMethods

	self compileAccessingMethodsFor: (self attributeDescriptions collect: [:each | each accessor selector]).
 .
	self attributeDescriptions do: [:each | 
		"self compileAccessingAndUpdatingMethodsForDescription: each."
		self compileMethodForDescription: each]! !

!SGModel class methodsFor: 'methods-creation' stamp: 'obi 6/13/2010 19:38'!
compileMethodForDescription: aDescription
	| newLine sourceString |
	newLine := Character lf asString.
	sourceString := WriteStream on: String new.
	sourceString nextPutAll: 'description',(self asLabel: aDescription accessor readSelector), newLine.
	sourceString nextPutAll: '	^', aDescription class name asString, ' new'; nextPutAll: newLine.
	sourceString nextPutAll: '		selectorAccessor: ''', aDescription accessor readSelector asString; nextPutAll: ''';'; nextPutAll: newLine.
	sourceString nextPutAll: '		cssClass: ''', aDescription kind name asString asLowercase; nextPutAll: ''';'; nextPutAll: newLine.
	aDescription properties keysAndValuesDo: [:each :aValue |
				self	methodPartFor: aDescription selector: each value: aValue lf: newLine on: sourceString].
	(aDescription isKindOf: MARelationDescription)
		ifTrue: [self addClassesAccessorFor: aDescription on: sourceString.
				"add default: to fix a magritte bug" 
				sourceString nextPutAll: newLine.
				sourceString nextPutAll: '		default: OrderedCollection new;'.
				sourceString nextPutAll: newLine].
	(aDescription isKindOf: SGTwowayRelationDescription)
		ifTrue: [sourceString nextPutAll: '		targetSelector: #', aDescription targetSelector printString.
				sourceString nextPutAll: newLine].
	sourceString nextPutAll: '		yourself'.
	^self compile: sourceString contents classified: #'accessing-description'
! !

!SGModel class methodsFor: 'attributes-accessing' stamp: 'obi 10/28/2008 13:19'!
createDefaultDescriptions
	self instVarNames do: [:aSymbol |
			self addDescription: (self defaultDescriptionOn: aSymbol)].
	^self attributeContainer children! !

!SGModel class methodsFor: 'migrating' stamp: 'obi 6/13/2010 18:09'!
createSubclass: aString instVarNames: anArrayOfStrings inDictionary: symbolDict
 "^self subclass: aString
		instVarNames: anArrayOfStrings
		classVars: #()
		classInstVars: #()
		poolDictionaries: #[]
		category: 'Generated Classes'
		inDictionary: symbolDict
		constraints: #[]
		instancesInvariant: false
		isModifiable: false"
		
^SGSystem createSubclass: aString of: self instVarNames: anArrayOfStrings inDictionary: symbolDict category: self defaultCategory! !

!SGModel class methodsFor: 'migrating' stamp: 'obi 6/13/2010 17:39'!
createUserSubclass: aString instVarNames: anArrayOfStrings
 "	| symbolDict |
	symbolDict := System myUserProfile symbolList objectNamed: #UserGlobals."
	^self createSubclass: aString instVarNames: anArrayOfStrings inDictionary: nil ! !

!SGModel class methodsFor: 'accessing' stamp: 'obi 11/04/2008 10:26'!
currentMandator
	^SGSystem currentMandator! !

!SGModel class methodsFor: 'accessing-report' stamp: 'obi 11/04/2008 11:58'!
defaultBatchSize
	^20! !

!SGModel class methodsFor: 'migrating' stamp: 'obi 6/13/2010 17:41'!
defaultCategory
	^'Generated Classes'! !

!SGModel class methodsFor: 'accessing-actions' stamp: 'obi 11/07/2008 07:06'!
defaultCommandColumn
	^SGCommandColumn new
			addCommand: [:comp :model | comp call: model open ] text: 'Open';
			addCommand: [:comp :model | 
				(comp lightbox: 
						(WAYesOrNoDialog new
							addMessage: 'Delete object: ', model displayString;
							yourself)) 
						ifTrue: [comp cache remove: model. model discard] ] text: 'Discard';
			yourself! !

!SGModel class methodsFor: 'accessing' stamp: 'obi 11/11/2008 13:31'!
defaultCssFor: aDescription
	^aDescription kind name asString! !

!SGModel class methodsFor: 'attributes-accessing' stamp: 'obi 10/31/2008 10:58'!
defaultDescriptionOn: aSelector
	| descSelector |
	descSelector := ('description',(self asLabel: aSelector)) asSymbol.
	^(self respondsTo: descSelector)
		ifTrue: [self perform: descSelector]
		ifFalse: [self stringDescriptionOn: aSelector]! !

!SGModel class methodsFor: 'accessing-actions' stamp: 'obi 11/04/2008 14:21'!
defaultEditActions
	^#()! !

!SGModel class methodsFor: 'accessing-actions' stamp: 'obi 11/05/2008 16:55'!
defaultListActions
	^Array with: #newEditWithTitle -> 'New'! !

!SGModel class methodsFor: 'accessing-actions' stamp: 'obi 11/08/2008 05:38'!
defaultOpenAction
	^#openWithTitle! !

!SGModel class methodsFor: 'accessing-report' stamp: 'obi 10/31/2008 15:09'!
defaultReport
	^self defaultReportOn: self allInstancesSorted! !

!SGModel class methodsFor: 'accessing-report' stamp: 'obi 11/05/2008 15:56'!
defaultReportOn: instances
	^self defaultReportOn: instances commands:  self defaultCommandColumn
! !

!SGModel class methodsFor: 'accessing-report' stamp: 'obi 05/12/2009 12:38'!
defaultReportOn: instances commands: commandColumn
	| report |
	report := MAReport new
				rowPeriod: 1;
				rows: instances;
				batchSize: self defaultBatchSize.
	self noneRelationDescriptions do: [:desc | report addColumnDescription: desc ].
	report addColumn: commandColumn.
	^report! !

!SGModel class methodsFor: 'accessing-actions' stamp: 'obi 11/08/2008 05:38'!
defaultShowAction
	^#showWithTitle! !

!SGModel class methodsFor: 'attributes-accessing' stamp: 'obi 10/28/2008 12:23'!
description
	"use cached container"
	^self attributeContainer! !

!SGModel class methodsFor: 'attributes-accessing' stamp: 'obi 11/03/2008 12:17'!
descriptionAt: anIndex put: anAttributeDescription
	^self attributeContainer children at: anIndex put: anAttributeDescription! !

!SGModel class methodsFor: 'attributes-accessing' stamp: 'obi 11/03/2008 12:00'!
descriptionContainer
	"Return the default description container."

	^ MAContainer new
		label: self label;
		yourself! !

!SGModel class methodsFor: 'attributes-accessing' stamp: 'obi 05/16/2009 10:49'!
descriptionForAttribute: anAttribute
	^self attributeContainer children detect: [:desc | desc accessor readSelector == anAttribute] ifNone: []! !

!SGModel class methodsFor: 'attributes-accessing' stamp: 'obi 11/03/2008 11:43'!
descriptionOn: aSelector type: aClass
	^aClass new
		cssClass: self name asString;
		label: (self asLabel: aSelector asString);
		priority: 1;
		selectorAccessor: aSelector;
		yourself.
! !

!SGModel class methodsFor: 'querying' stamp: 'obi 10/26/2008 01:01'!
detect: aBlock
	^self detect: aBlock ifNone: []! !

!SGModel class methodsFor: 'querying' stamp: 'obi 10/26/2008 01:01'!
detect: aBlock ifNone: otherBlock
	^self allInstances detect: aBlock ifNone: otherBlock! !

!SGModel class methodsFor: 'querying' stamp: 'obi 10/26/2008 01:02'!
do: aBlock
	^self allInstances do: aBlock! !

!SGModel class methodsFor: 'public' stamp: 'obi 11/01/2008 10:27'!
edit
	^SGEditClassPage on: self! !

!SGModel class methodsFor: 'accessing' stamp: 'obi 11/07/2008 14:36'!
editorClass
	^SGSystem classNamed: self editorClassSymbol! !

!SGModel class methodsFor: 'accessing' stamp: 'obi 11/07/2008 14:36'!
editorClassSymbol
	^(name, 'EditorComponent') asSymbol! !

!SGModel class methodsFor: 'attributes-accessing' stamp: 'obi 11/04/2008 12:14'!
fullDescription
	| newContainer |
	newContainer := self descriptionContainer.
	newContainer setChildren: self allAttributeDescriptions.
	^newContainer
	! !

!SGModel class methodsFor: 'initialization' stamp: 'obi 10/31/2008 11:33'!
initialize
	self withAllSubclassesDo: [:cl | cl isBasic ifTrue: [cl resetDescriptions]]! !

!SGModel class methodsFor: 'accessing' stamp: 'obi 11/01/2008 12:16'!
instancesSize
	^self allInstances size! !

!SGModel class methodsFor: 'attributes-accessing' stamp: 'obi 11/11/2008 13:33'!
integerDescriptionOn: aSelector
	^MANumberDescription new
		cssClass: 'number';
		label: (self asLabel: aSelector asString);
		priority: 1;
		default: 0;
		selectorAccessor: aSelector;
		yourself.
! !

!SGModel class methodsFor: 'testing' stamp: 'obi 11/02/2008 11:18'!
isBasic
	^name == #SGModel! !

!SGModel class methodsFor: 'accessing' stamp: 'obi 10/31/2008 15:07'!
keyAttribute
	^self allInstVarNames first! !

!SGModel class methodsFor: 'accessing' stamp: 'obi 11/08/2008 05:04'!
listClass
	^SGSystem classNamed: self listClassSymbol! !

!SGModel class methodsFor: 'accessing' stamp: 'obi 11/08/2008 05:04'!
listClassSymbol
	^(name, 'ListComponent') asSymbol! !

!SGModel class methodsFor: 'attributes-accessing' stamp: 'obi 11/11/2008 13:32'!
memoDescriptionOn: aSelector
	^MAMemoDescription new
		cssClass: 'string';
		label: (self asLabel: aSelector asString);
		priority: 1;
		selectorAccessor: aSelector;
		yourself.
! !

!SGModel class methodsFor: 'accessing' stamp: 'obi 6/13/2010 16:22'!
meta
	Meta ifNil: [Meta := Dictionary new].
	^Meta! !

!SGModel class methodsFor: 'accessing' stamp: 'go 10/25/2008 10:33'!
meta: aMetaDict
	Meta := aMetaDict! !

!SGModel class methodsFor: 'meta-accessing' stamp: 'obi 10/28/2008 11:39'!
metaAtKey: aKey ifAbsent: aBlock
	^self meta at: aKey ifAbsent: aBlock! !

!SGModel class methodsFor: 'meta-accessing' stamp: 'obi 10/28/2008 12:42'!
metaAtKey: aKey put: anObject
	^self meta at: aKey put: anObject! !

!SGModel class methodsFor: 'methods-creation' stamp: 'obi 6/13/2010 19:39'!
methodPartFor: aDescription selector: aSelector value: aValue lf: newLine on: sourceString
	aValue isNil
			ifFalse: [
				((aValue isKindOf: Collection) not or: [aValue isKindOf: String])
					ifTrue: [sourceString nextPutAll: '		', aSelector asString,': ', aValue  printString; nextPutAll: ';'; nextPutAll: newLine]].! !

!SGModel class methodsFor: 'migrating' stamp: 'obi 11/01/2008 11:13'!
migrateInstances: instances to: anotherClass

"Override to enable commits during migration of large collections"
"extra commits are not allowed atm"


! !

!SGModel class methodsFor: 'private' stamp: 'obi 6/13/2010 20:19'!
mySuperClasses

"Returns an Array of all superclasses up to SGModel"

| result currClass |
result:= OrderedCollection new.
(self name == #SGModel)
  ifTrue: [^ result]
  ifFalse: [
    currClass := self superclass.
    [ currClass == SGModel ] whileFalse: [
      result add: currClass.
      currClass := currClass superclass.
    ].
  ].
^result
! !

!SGModel class methodsFor: 'instance creation' stamp: 'obi 10/31/2008 15:28'!
new
	| newObject |
	newObject := super new.
	self add: newObject.
	^newObject! !

!SGModel class methodsFor: 'public' stamp: 'obi 11/04/2008 12:02'!
newEdit
	^self newOpen! !

!SGModel class methodsFor: 'public' stamp: 'obi 11/05/2008 16:57'!
newEditWithTitle
	^self new openWithTitle! !

!SGModel class methodsFor: 'public' stamp: 'obi 10/31/2008 15:41'!
newOpen
	^self new open! !

!SGModel class methodsFor: 'accessing' stamp: 'obi 11/07/2008 13:43'!
noneBasicClasses
	^self allClasses select: [:cl | cl isBasic not]! !

!SGModel class methodsFor: 'attributes-accessing' stamp: 'obi 05/12/2009 12:38'!
noneRelationDescriptions
	^self allAttributeDescriptions select: [:desc | desc kind ~~ Collection]! !

!SGModel class methodsFor: 'attributes-accessing' stamp: 'obi 11/11/2008 13:31'!
passwordDescriptionOn: aSelector
	^MAPasswordDescription new
		cssClass: 'string';
		label: (self asLabel: aSelector asString);
		priority: 1;
		selectorAccessor: aSelector;
		yourself.
! !

!SGModel class methodsFor: 'accessing-actions' stamp: 'obi 11/08/2008 10:27'!
projectCommandColumn
	^SGCommandColumn new
			addCommand: [:comp :model | WARenderLoop new call: model showWithTitle ] text: 'Show';
			addCommand: [:comp :model | WARenderLoop new call: model openWithTitle ] text: 'Edit';
			addCommand: [:comp :model | 
				(comp lightbox: (WAYesOrNoDialog new
						addMessage: 'Delete object: ', model displayString;
						yourself) ) 
						ifTrue: [comp cache remove: model. model discard] ] text: 'Discard';
			yourself! !

!SGModel class methodsFor: 'accessing-report' stamp: 'obi 11/05/2008 15:57'!
projectReport
	^self projectReportOn: self allInstancesSorted! !

!SGModel class methodsFor: 'accessing-report' stamp: 'obi 11/05/2008 15:58'!
projectReportOn: instances
	^self defaultReportOn: instances commands:  self projectCommandColumn
! !

!SGModel class methodsFor: 'attributes-accessing' stamp: 'obi 11/11/2008 13:33'!
relationDescriptionOn: aSelector
	^MAToManyRelationDescription new
		label: (self asLabel: aSelector asString);
		priority: 1;
		selectorAccessor: aSelector;
		yourself.
! !

!SGModel class methodsFor: 'adding-removing' stamp: 'obi 12/20/2008 12:35'!
remove: anObject
	^self currentMandator remove: anObject! !

!SGModel class methodsFor: 'accessing' stamp: 'obi 11/03/2008 12:03'!
removeAttributeContainer
	self meta removeKey: #attributeContainer ifAbsent: []! !

!SGModel class methodsFor: 'attributes-accessing' stamp: 'obi 11/03/2008 11:45'!
removeDescription: anAttributeDescription
	^self attributeContainer remove: anAttributeDescription! !

!SGModel class methodsFor: 'attributes-accessing' stamp: 'obi 11/03/2008 12:04'!
resetDescriptions
	self removeAttributeContainer.
	self setDescriptions: OrderedCollection new.
	^self updateDescriptions
! !

!SGModel class methodsFor: 'querying' stamp: 'obi 10/26/2008 01:03'!
safelyDo: aBlock
	^self allInstances copy do: aBlock! !

!SGModel class methodsFor: 'querying' stamp: 'obi 10/26/2008 01:01'!
select: aBlock
	^self allInstances select: aBlock! !

!SGModel class methodsFor: 'attributes-accessing' stamp: 'obi 10/28/2008 12:19'!
setDescriptions: descriptionColl
	self attributeContainer setChildren: descriptionColl! !

!SGModel class methodsFor: 'attributes-accessing' stamp: 'obi 11/11/2008 13:31'!
stringDescriptionOn: aSelector
	^MAStringDescription new
		cssClass: 'string';
		label: (self asLabel: aSelector asString);
		priority: 1;
		selectorAccessor: aSelector;
		yourself.
! !

!SGModel class methodsFor: 'migrating' stamp: 'obi 11/04/2008 10:57'!
subclass: aString instVarNames: anArrayOfStrings

	^SGSystem redefineClassNamed: aString with: [
		self createUserSubclass: aString instVarNames: anArrayOfStrings ].! !

!SGModel class methodsFor: 'attributes-accessing' stamp: 'obi 11/01/2008 11:44'!
updateDescriptions
	| currentDescriptions theInstVars |
	currentDescriptions := self attributeDescriptions.
	theInstVars := self instVarNames.
	theInstVars do: [:aSymbol |
		(currentDescriptions detect: [:desc | desc accessor readSelector == aSymbol] ifNone: []) isNil
			ifTrue: [self addDescription: (self defaultDescriptionOn: aSymbol)]].
	currentDescriptions copy do: [:desc |
		(theInstVars includes: desc accessor readSelector)
			ifFalse: [self attributeContainer remove: desc] ].
	^self attributeDescriptions
! !

!SGModel class methodsFor: 'accessing' stamp: 'obi 11/08/2008 01:10'!
viewClass
	^SGSystem classNamed: self viewClassSymbol! !

!SGModel class methodsFor: 'accessing' stamp: 'obi 11/08/2008 01:10'!
viewClassSymbol
	^(name, 'ViewComponent') asSymbol! !

!SGModel methodsFor: 'accessing' stamp: 'obi 05/05/2009 12:56'!
accessorFor: anAttribute
	^self class accessorFor: anAttribute! !

!SGModel methodsFor: 'accessing' stamp: 'obi 01/02/2009 10:22'!
asComponent
	^ self class fullDescription asComponentOn: self! !

!SGModel methodsFor: 'printing' stamp: 'obi 12/29/2008 12:21'!
asString
	^self displayString! !

!SGModel methodsFor: 'accessing' stamp: 'obi 10/31/2008 11:43'!
attributeDescriptions
	^self class attributeDescriptions! !

!SGModel methodsFor: 'removing' stamp: 'obi 05/05/2009 12:44'!
basicDiscard
	self release.
	self class remove: self! !

!SGModel methodsFor: 'testing' stamp: 'obi 10/31/2008 15:34'!
canDiscard
	^true! !

!SGModel methodsFor: 'removing' stamp: 'obi 10/31/2008 15:41'!
discard
	self canDiscard
		ifTrue: [self basicDiscard]! !

!SGModel methodsFor: 'printing' stamp: 'obi 11/25/2008 14:20'!
displayName
	^(self instVarAt: 1) asString! !

!SGModel methodsFor: 'printing' stamp: 'obi 11/04/2008 11:56'!
displayString
	^(self instVarAt: 1) asString , ' (', self class name asString,')'! !

!SGModel methodsFor: 'initialize-release' stamp: 'obi 03/27/2009 11:18'!
initialize
	"some hacks to avoid using always the same collection"
	self class attributeDescriptions do: [:desc | 
		desc kind = Collection
			ifTrue: [desc accessor write: OrderedCollection new to: self ]
			ifFalse: [self initializeWith: desc accessor using: desc]].
	self postInitialize
	! !

!SGModel methodsFor: 'initialize-release' stamp: 'obi 11/07/2008 05:56'!
initializeWith: maAccessor using: aDescription
"only init nil values"
	(maAccessor canWrite: self)
		ifTrue: [ maAccessor write: aDescription default to: self ] 
	! !

!SGModel methodsFor: 'initialize-release' stamp: 'obi 05/17/2009 11:50'!
open
	^self asComponent addValidatedForm! !

!SGModel methodsFor: 'actions' stamp: 'obi 11/08/2008 10:37'!
openWithTitle
	^self class editorClass on: self! !

!SGModel methodsFor: 'post-actions' stamp: 'obi 11/07/2008 06:14'!
postActions
	"override in subclasses if you need some one"! !

!SGModel methodsFor: 'post-actions' stamp: 'obi 11/07/2008 06:15'!
postInitialize
	"override in subclasses if you need some one"! !

!SGModel methodsFor: 'initialize-release' stamp: 'obi 05/05/2009 12:45'!
release
	"release all instVars"
	self class allAttributeDescriptions do: [:desc | desc accessor write: nil to: self ].

	! !

!SGModel methodsFor: 'initialize-release' stamp: 'obi 11/07/2008 05:57'!
safeInitialize
"only init nil values"
	self class attributeDescriptions do: [:desc | | maAccessor |
			maAccessor := desc accessor.
			(self perform: maAccessor readSelector)
				isNil ifTrue: [self initializeWith: maAccessor using: desc] ]
	! !

!SGModel methodsFor: 'actions' stamp: 'obi 11/05/2008 16:07'!
show
	^self asComponent addValidatedSwitch! !

!SGModel methodsFor: 'actions' stamp: 'obi 11/08/2008 10:37'!
showWithTitle
	^ self class viewClass on: self
! !

SGModel subclass: #SGProject
	instanceVariableNames: 'name path description modelClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Scaffolding-Model'!

!SGProject class methodsFor: 'accessing' stamp: 'obi 11/05/2008 10:34'!
currentMandator
	"always use the default mandator"
	^SGMandator default! !

!SGProject class methodsFor: 'descriptions' stamp: 'obi 11/05/2008 10:30'!
descriptionDescription
	| desc |
	desc := self memoDescriptionOn: #description.
	^desc! !

!SGProject class methodsFor: 'descriptions' stamp: 'obi 11/05/2008 11:20'!
descriptionModelClass
	| desc |
	desc := self stringDescriptionOn: #modelClass.
	desc required: true.
	^desc! !

!SGProject class methodsFor: 'descriptions' stamp: 'obi 11/05/2008 10:28'!
descriptionName
	| desc |
	desc := self stringDescriptionOn: #name.
	desc required: true.
	^desc! !

!SGProject class methodsFor: 'descriptions' stamp: 'obi 11/07/2008 07:28'!
descriptionPath
	| desc |
	desc := self stringDescriptionOn: #path.
	desc default: ''.
	^desc! !

!SGProject class methodsFor: 'testing' stamp: 'obi 11/05/2008 10:31'!
isBasic
	^true! !

!SGProject methodsFor: 'private' stamp: 'obi 11/07/2008 07:21'!
application
	^WADispatcher default entryPointAt: self entryPoint! !

!SGProject methodsFor: 'actions' stamp: 'obi 12/29/2008 12:39'!
basicDiscard
	| anApplication |
	modelClass isEmpty
		ifFalse: [anApplication := self application.
				anApplication isNil ifFalse: [WADispatcher default unregister: anApplication] ].
	super basicDiscard.
	! !

!SGProject methodsFor: 'Accessing' stamp: 'obi 11/25/2008 14:29'!
componentClass
	^SGSystem classNamed: self componentClassName asSymbol
! !

!SGProject methodsFor: 'Accessing' stamp: 'obi 11/25/2008 14:29'!
componentClassName
	^modelClass ,'Component'
! !

!SGProject methodsFor: 'Accessing' stamp: 'obi 11/05/2008 10:51'!
description
	^description 
! !

!SGProject methodsFor: 'Updating' stamp: 'obi 11/05/2008 10:51'!
description: newValue

   "Modify the value of the instance variable 'description'."
   description := newValue
! !

!SGProject methodsFor: 'private' stamp: 'obi 11/08/2008 09:36'!
entryPoint
	| aPath |
	aPath := self path.
	aPath isEmpty ifTrue: [^'/',self name].
	^(aPath first == $/ )
		ifTrue: [(aPath size > 1 ifTrue: [aPath,'/'] ifFalse: [aPath]), self name]
		ifFalse: ['/', aPath, '/', self name]
		! !

!SGProject methodsFor: 'Accessing' stamp: 'obi 11/05/2008 11:26'!
modelClass
	^modelClass 
! !

!SGProject methodsFor: 'Updating' stamp: 'obi 11/05/2008 11:26'!
modelClass: newValue

   "Modify the value of the instance variable 'modelClass'."
   modelClass := newValue
! !

!SGProject methodsFor: 'Accessing' stamp: 'obi 11/05/2008 10:51'!
name
	^name 
! !

!SGProject methodsFor: 'Updating' stamp: 'obi 11/05/2008 10:51'!
name: newValue

   "Modify the value of the instance variable 'name'."
   name := newValue
! !

!SGProject methodsFor: 'actions' stamp: 'obi 11/05/2008 10:48'!
open
	^SGProjectPage on: self! !

!SGProject methodsFor: 'Accessing' stamp: 'obi 11/07/2008 07:30'!
path
	path ifNil: [path := ''].
	^path! !

!SGProject methodsFor: 'Updating' stamp: 'obi 11/07/2008 07:27'!
path: newValue

   "Modify the value of the instance variable 'path'."
   path := newValue
! !

!SGProject methodsFor: 'actions' stamp: 'obi 11/08/2008 00:35'!
save
	modelClass isEmpty
		ifFalse: [SGSystem createProjectFor: self named: name]! !

SGModel subclass: #SGUser
	instanceVariableNames: 'loginName lastName firstName password email defaultMandator allowedMandators groups'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Scaffolding-Model'!

!SGUser class methodsFor: 'accessing-description' stamp: 'obi 6/13/2010 20:10'!
descriptionAllowedMandators
	^MAToManyRelationDescription new
		selectorAccessor: 'allowedMandators';
		cssClass: 'collection';
		definitive: true;
		priority: 100;
		ordered: true;
		label: 'Allowed';
		visible: false;
		classes: (OrderedCollection with: SGMandator);
		default: OrderedCollection new;
		yourself! !

!SGUser class methodsFor: 'descriptions' stamp: 'obi 12/29/2008 12:08'!
descriptionDefaultMandator
	^SGSingleOptionDescription new
		options: [SGMandator allInstancesSorted];
		accessor: #defaultMandator;
		label: 'Default';
		default: SGMandator default;
		priority: 100;
		visible: true;
		yourself! !

!SGUser class methodsFor: 'accessing-description' stamp: 'obi 05/05/2009 13:14'!
descriptionEmail
	^MAStringDescription new
		selectorAccessor: 'email';
		cssClass: 'string';
		priority: 1;
		label: 'Email';
		yourself! !

!SGUser class methodsFor: 'accessing-description' stamp: 'obi 05/05/2009 13:14'!
descriptionFirstName
	^MAStringDescription new
		selectorAccessor: 'firstName';
		cssClass: 'string';
		priority: 1;
		label: 'FirstName';
		yourself! !

!SGUser class methodsFor: 'accessing-description' stamp: 'obi 6/13/2010 20:08'!
descriptionGroups
	^SGTwowayRelationDescription new
		selectorAccessor: 'groups';
		targetSelector: #users;
		cssClass: 'object';
		priority: 100;
		label: 'Groups';
		visible: true;
		classes: (OrderedCollection with: SGGroup);
		yourself! !

!SGUser class methodsFor: 'accessing-description' stamp: 'obi 05/05/2009 13:14'!
descriptionLastName
	^MAStringDescription new
		selectorAccessor: 'lastName';
		cssClass: 'string';
		priority: 1;
		label: 'LastName';
		yourself! !

!SGUser class methodsFor: 'accessing-description' stamp: 'obi 05/05/2009 13:14'!
descriptionLoginName
	^MAStringDescription new
		selectorAccessor: 'loginName';
		cssClass: 'string';
		required: true;
		priority: 1;
		label: 'LoginName';
		yourself! !

!SGUser class methodsFor: 'accessing-description' stamp: 'obi 05/05/2009 13:14'!
descriptionPassword
	^MAPasswordDescription new
		selectorAccessor: 'password';
		cssClass: 'string';
		required: true;
		priority: 1;
		label: 'Password';
		yourself! !

!SGUser class methodsFor: 'accessing' stamp: 'obi 10/31/2008 09:32'!
isBasic
	^name == #SGUser! !

!SGUser methodsFor: 'accessing' stamp: 'obi 04/19/2009 10:51'!
allowedMandators

   "Return the value of the instance variable 'allowedMandators'."
   ^allowedMandators
! !

!SGUser methodsFor: 'updating' stamp: 'obi 04/19/2009 10:51'!
allowedMandators: newValue

   "Modify the value of the instance variable 'allowedMandators'."
   allowedMandators := newValue
! !

!SGUser methodsFor: 'accessing' stamp: 'obi 04/19/2009 10:51'!
defaultMandator

   "Return the value of the instance variable 'defaultMandator'."
   ^defaultMandator
! !

!SGUser methodsFor: 'updating' stamp: 'obi 04/19/2009 10:51'!
defaultMandator: newValue

   "Modify the value of the instance variable 'defaultMandator'."
   defaultMandator := newValue
! !

!SGUser methodsFor: 'accessing' stamp: 'obi 04/19/2009 10:51'!
email

   "Return the value of the instance variable 'email'."
   ^email
! !

!SGUser methodsFor: 'updating' stamp: 'obi 04/19/2009 10:51'!
email: newValue

   "Modify the value of the instance variable 'email'."
   email := newValue
! !

!SGUser methodsFor: 'accessing' stamp: 'obi 04/19/2009 10:51'!
firstName

   "Return the value of the instance variable 'firstName'."
   ^firstName
! !

!SGUser methodsFor: 'updating' stamp: 'obi 04/19/2009 10:51'!
firstName: newValue

   "Modify the value of the instance variable 'firstName'."
   firstName := newValue
! !

!SGUser methodsFor: 'accessing' stamp: 'obi 04/19/2009 10:48'!
groups

   "Return the value of the instance variable 'groups'."
   ^groups
! !

!SGUser methodsFor: 'updating' stamp: 'obi 04/19/2009 10:48'!
groups: newValue

   "Modify the value of the instance variable 'groups'."
   groups := newValue
! !

!SGUser methodsFor: 'updating' stamp: 'obi 05/05/2009 13:14'!
groupsAdd: aModel

	(self accessorFor: #groups) write: aModel to: self! !

!SGUser methodsFor: 'updating' stamp: 'obi 05/05/2009 13:14'!
groupsRemove: aModel

	(self accessorFor: #groups) remove: aModel from: self! !

!SGUser methodsFor: 'accessing' stamp: 'obi 04/19/2009 10:51'!
lastName

   "Return the value of the instance variable 'lastName'."
   ^lastName
! !

!SGUser methodsFor: 'updating' stamp: 'obi 04/19/2009 10:51'!
lastName: newValue

   "Modify the value of the instance variable 'lastName'."
   lastName := newValue
! !

!SGUser methodsFor: 'accessing' stamp: 'obi 04/19/2009 10:51'!
loginName

   "Return the value of the instance variable 'loginName'."
   ^loginName
! !

!SGUser methodsFor: 'updating' stamp: 'obi 04/19/2009 10:51'!
loginName: newValue

   "Modify the value of the instance variable 'loginName'."
   loginName := newValue
! !

!SGUser methodsFor: 'accessing' stamp: 'obi 04/19/2009 10:51'!
password

   "Return the value of the instance variable 'password'."
   ^password
! !

!SGUser methodsFor: 'updating' stamp: 'obi 04/19/2009 10:51'!
password: newValue

   "Modify the value of the instance variable 'password'."
   password := newValue
! !

Object subclass: #SGSystem
	instanceVariableNames: ''
	classVariableNames: 'Current Mandator Types Users'
	poolDictionaries: ''
	category: 'Seaside-Scaffolding'!

!SGSystem class methodsFor: 'accessing-types' stamp: 'obi 10/26/2008 12:25'!
addType: aType
	self types add: aType! !

!SGSystem class methodsFor: 'class utilities' stamp: 'obi 6/13/2010 16:26'!
classNamed: aName
	^Smalltalk classNamed: aName
	"System myUserProfile symbolList objectNamed: aName"! !

!SGSystem class methodsFor: 'migrating' stamp: 'obi 6/13/2010 18:07'!
compile: sourceString intoClass: aClass classified: aSymbol
	^GRPlatform current
			compile: sourceString
			into: aClass
			classified: aSymbol! !

!SGSystem class methodsFor: 'migrating' stamp: 'obi 6/13/2010 19:01'!
copyMethodsFrom: originalClass to: newClass dictionaries: ns

	originalClass selectors do: [:aSelector |
		  newClass copy: aSelector from: originalClass classified: (originalClass categoryForSelector: aSelector)].
	originalClass class selectors do: [:aSelector |
		  newClass class copy: aSelector from: originalClass class classified: (originalClass class categoryForSelector: aSelector)]
! !

!SGSystem class methodsFor: 'private-project' stamp: 'obi 6/13/2010 18:10'!
createComponentClassNamed: className superclass: aSuperClass
	| aClass |
	aClass := self classNamed: className.
	aClass isNil
		ifTrue: [ aClass := self createSubclass: className of: aSuperClass instVarNames: #() inDictionary: nil category: self projectCategory ]. 	^aClass! !

!SGSystem class methodsFor: 'public-project' stamp: 'obi 6/13/2010 18:02'!
createProjectFor: aProject named: aName
	| aModelClassName aClass aModelClass componentClasses |
	aModelClassName := aProject modelClass.
	aModelClass := self classNamed: aModelClassName.
	
	"create class for my project component"
	aClass := self createComponentClassNamed: aModelClassName, 'Component' superclass: SGProjectComponent.
	componentClasses := OrderedCollection with: aClass.
	
	"create class for my editor component"
	componentClasses add: (self createComponentClassNamed: aModelClass editorClassSymbol superclass: SGEditorComponent).
	
	"create class for my view component"
	componentClasses add: (self createComponentClassNamed: aModelClass viewClassSymbol superclass: SGViewComponent).
	
	"create class for my list component"
	componentClasses add: (self createComponentClassNamed: aModelClass listClassSymbol superclass: SGListComponent).
	
	"register my application"
	(WAAdmin register: aClass asApplicationAt: aProject entryPoint)
		addLibrary: PTDeploymentLibrary;
		addLibrary: SUDeploymentLibrary;
		addLibrary: SGProjectLibrary;
	     	preferenceAt: #sessionClass put: SGSession;
		yourself.
		
	"create default methods for each component class"
	self createProjectMethodsFor: aModelClass on: componentClasses! !

!SGSystem class methodsFor: 'public-project' stamp: 'obi 6/13/2010 18:11'!
createProjectMethodsFor: aModelClass on: componentClasses 
	| newLine |
	newLine := Character lf asString.
	componentClasses do: [:aComponentClass | 
			self compile: 'modelClass',newLine, '	^', aModelClass name asString
				intoClass: aComponentClass class
				classified: #accessing]
! !

!SGSystem class methodsFor: 'migrating' stamp: 'obi 6/13/2010 18:09'!
createSubclass: aString of: superClass instVarNames: anArrayOfStrings inDictionary: symbolDict category: cat
 "^self subclass: aString
		instVarNames: anArrayOfStrings
		classVars: #()
		classInstVars: #()
		poolDictionaries: #[]
		category: 'Generated Classes'
		inDictionary: symbolDict
		constraints: #[]
		instancesInvariant: false
		isModifiable: false"
		
^superClass subclass: aString
	instanceVariableNames: anArrayOfStrings
	classVariableNames: ''
	poolDictionaries: ''
	category: cat! !

!SGSystem class methodsFor: 'accessing' stamp: 'dkh 04/08/2009 11:15'!
currentMandator
	|aSession |
	aSession := WACurrentSession value.	
	^aSession == nil
		ifFalse: [aSession mandator]
		ifTrue: [
			Mandator == nil 
				ifTrue: [ SGMandator default ]
				ifFalse: [ Mandator ]]! !

!SGSystem class methodsFor: 'accessing' stamp: 'obi 11/04/2008 10:26'!
currentMandator: aMandator
	| aSession |
	aSession := WACurrentSession value.
	aSession isNil 
		ifTrue: [Mandator := aMandator]
		ifFalse: [aSession mandator: aMandator]! !

!SGSystem class methodsFor: 'class utilities' stamp: 'obi 10/26/2008 11:11'!
hasInstancesOf: className
	SGMandator allDo: [:aMandator | (aMandator hasInstancesOf: className) ifTrue: [^true] ].
	^false! !

!SGSystem class methodsFor: 'accessing' stamp: 'obi 10/29/2008 14:59'!
mandators
	^SGMandator list! !

!SGSystem class methodsFor: 'migrating' stamp: 'obi 6/13/2010 16:24'!
migrateInstancesWithSubclassesOf: topClass

	"Get the instances from all Mandators"
	"Migrate them all."
	"Do commits during migration of more than 10000 instances"
	"commit is currently not allowed inside a seaside transaction !!"
	
	"GemStone only"! !

!SGSystem class methodsFor: 'class utilities' stamp: 'obi 10/29/2008 14:59'!
modelClasses
	^SGModel withAllSubclasses! !

!SGSystem class methodsFor: 'accessing' stamp: 'obi 6/13/2010 18:53'!
namespace
	"used for GemStone compatibility"
	"GsSession currentSession symbolList."
	^nil ! !

!SGSystem class methodsFor: 'private-project' stamp: 'obi 11/04/2008 13:22'!
projectCategory
	^'GemStone-Project'! !

!SGSystem class methodsFor: 'migrating' stamp: 'obi 6/13/2010 19:02'!
redefineClassNamed: aClassName with: aBlock
	"after redefining the class migrate self and all subclasses and also do a recompile of my accessors and my descriptions"
	"remove also the old class versions afterwards"
	| originalClass classOrganizer cls |
	originalClass := self classNamed: aClassName asSymbol.
	originalClass ~~ nil 
		ifTrue: [ 
			"pre-create the classOrganizer, so that original subclasses are preserved."
			"classOrganizer := ClassOrganizer cachedOrganizer "].
	cls := aBlock value.
	cls == nil
		ifTrue: [ ^nil ].
	
	originalClass ~~ nil
		ifTrue: [ self copyMethodsFrom: originalClass to: cls	dictionaries: self namespace.
			"copy meta data"
			 cls meta: originalClass meta.
			cls comment: originalClass comment.
			"force redefinition of existing subclasses"
			self redefineSubclassesOf: originalClass using: classOrganizer.
			self migrateInstancesWithSubclassesOf: cls.
			"cls classHistory removeVersion: originalClass"].
	
	"create accessors by default, but delete old ones"
	(cls allMethodsInCategory: #accessing) do: [:m | cls removeSelector: m].
	(cls allMethodsInCategory: #updating) do: [:m | cls removeSelector: m].
	cls updateDescriptions.
	cls compileDescriptionMethods.
	^cls! !

!SGSystem class methodsFor: 'migrating' stamp: 'obi 6/13/2010 19:08'!
redefineSubclassesOf: aClass using: aClassOrganizer
"GemStone only ?"
"	(aClassOrganizer subclassesOf: aClass) do: [:subClass | | newClass |
		newClass := (self classNamed: subClass superclass name) createUserSubclass: subClass name instVarNames: subClass instVarNames.
		newClass 
			copyMethodsFrom: subClass 
			dictionaries: self namespace .
		newClass meta: subClass meta.
		newClass comment: subClass comment.
		self redefineSubclassesOf: subClass using: aClassOrganizer ]."! !

!SGSystem class methodsFor: 'class utilities' stamp: 'obi 11/01/2008 11:26'!
removeClassNamed: aClassName
	^self removeClassNamed: aClassName check: true! !

!SGSystem class methodsFor: 'class utilities' stamp: 'obi 6/13/2010 18:54'!
removeClassNamed: aClassName check: aBoolean
	"the class should not have any instances and no subclasses"
	| aClass |
	aClass := self classNamed: aClassName asSymbol.
	aClass isNil 
		ifFalse: [
			aBoolean ifTrue: [aClass canDelete ifFalse: [^false]].
			aClass removeFromSystem.
			SGMandator allDo: [:aMandator | aMandator removeClassNamed: aClassName]].
	^true! !

!SGSystem class methodsFor: 'accessing' stamp: 'obi 11/04/2008 07:38'!
sessionStateNumber
	^25! !

!SGSystem class methodsFor: 'accessing' stamp: 'obi 10/26/2008 12:26'!
types
	Types isNil ifTrue: [Types := IdentityBag new].
	^Types! !

!SGSystem class methodsFor: 'accessing' stamp: 'obi 10/29/2008 15:02'!
users
	^SGUser allInstances! !

!MADescription methodsFor: '*seaside-scaffolding-method-creation' stamp: 'obi 05/05/2009 12:59'!
accessingMethods
	^#()! !

!MADescription methodsFor: '*seaside-scaffolding-method-creation' stamp: 'obi 05/05/2009 12:59'!
updatingMethods
	^#()! !
SGModel initialize!
